#include "neogeo.h"

int bSaveDecryptedCs = 0;
int bSaveDecryptedPs = 0;
int bSaveDecryptedS1 = 0;
int bSaveDecryptedVs = 0;
int bSaveDecryptedM1 = 0;

// 8 C-ROMs for standard ROMsets, number will vary
void SaveDecCROMs()
{
	extern UINT32 nSpriteSize;

	if (NeoSpriteROM == NULL) {
		return;
	}

	TCHAR szNames[64] = _T("");
	char name[64] = "";
	int divby = 0x1000000;

	BurnUpdateProgress(0.0, _T("Initializing save routine..."), 0);

	if (!(nSpriteSize & 0xFFFFFF)) {
		if (!strcmp(BurnDrvGetTextA(DRV_NAME), "svcpcb")) divby = 0x4000000;
		else if (!strcmp(BurnDrvGetTextA(DRV_NAME), "svcpcba") || !strcmp(BurnDrvGetTextA(DRV_NAME), "kf2k3pcb") || !strcmp(BurnDrvGetTextA(DRV_NAME), "ms5pcb")) divby = 0x2000000;

		for (unsigned int i = 0; i < (nSpriteSize / divby); i++) {
			_stprintf(szNames, _T("Saving Decrypted C%d and C%d ROMs..."), ((i * 2) + 1), ((i * 2) + 2));
			BurnUpdateProgress(0.0, szNames, 0);

			sprintf(name, "%X%2.2X-c%d.bin", Neo68KROMActive[0x109], Neo68KROMActive[0x108], ((i * 2) + 1));
			FILE* C1 = fopen(name, "wb");
			sprintf(name, "%X%2.2X-c%d.bin", Neo68KROMActive[0x109], Neo68KROMActive[0x108], ((i * 2) + 2));
			FILE* C2 = fopen(name, "wb");

			if (C1 && C2) {
				for (int j = 0; j < divby; j += 2) {
					fwrite(NeoSpriteROM + i * divby + j + 0, 1, 1, C1);
					fwrite(NeoSpriteROM + i * divby + j + 1, 1, 1, C2);
				}

				fclose(C1);
				fclose(C2);
			}
		}

		return;
	}
}

// For compatibility issues, only 1 P-ROM will be saved
void SaveDecPROM()
{
	extern UINT32 nCodeSize;

	char name[64] = "";

	BurnUpdateProgress(0.0, _T("Saving Decrypted P-ROM..."), 0);

	sprintf (name, "%X%2.2X-p1d.bin", Neo68KROMActive[0x109], Neo68KROMActive[0x108]);
	FILE* file = fopen(name, "wb");

	if (file) {
		fwrite(Neo68KROMActive, 1, nCodeSize, file);
		fclose(file);
	}
}

// Either extracted from C Data or descrambled text roms
void SaveDecSROM()
{
	char name[64] = "";

	BurnUpdateProgress(0.0, _T("Saving Decrypted S1-ROM..."), 0);

	sprintf (name, "%X%2.2X-s1d.bin", Neo68KROMActive[0x109], Neo68KROMActive[0x108]);
	FILE* file = fopen(name, "wb");

	if (file) {
		fwrite(NeoTextROM + 0x020000, 1, nNeoTextROMSize, file);
		fclose(file);
	}
}

// Standard decrypted samples are saved with this
void SaveDecVROMs(int nNumber)
{
	extern UINT8* YM2610ADPCMAROM;
	extern INT32 nYM2610ADPCMASize;

	char name[64] = "";

	// Special handler for unique V-ROM
	if (nNumber == 1) {
		BurnUpdateProgress(0.0, _T("Saving Decrypted V1 ROM..."), 0);

		sprintf (name, "%X%2.2X-v1d.bin", Neo68KROMActive[0x109], Neo68KROMActive[0x108]);
		FILE* V1ROM = fopen(name, "wb");

		if (V1ROM) {
			fwrite(YM2610ADPCMAROM, 1, nYM2610ADPCMASize, V1ROM);
			fclose(V1ROM);
		}
	} else if (nNumber == 3) {
		BurnUpdateProgress(0.0, _T("Saving Decrypted V1 V2 V3 ROM..."), 0);

		sprintf (name, "%X%2.2X-v1d.bin", Neo68KROMActive[0x109], Neo68KROMActive[0x108]);
		FILE* V1ROM = fopen(name, "wb");

		if (V1ROM) {
			fwrite(YM2610ADPCMAROM, 1, 0x400000, V1ROM);
			fclose(V1ROM);
		}

		sprintf (name, "%X%2.2X-v2d.bin", Neo68KROMActive[0x109], Neo68KROMActive[0x108]);
		FILE* V2ROM = fopen(name, "wb");

		if (V2ROM) {
			fwrite(YM2610ADPCMAROM + 0x400000, 1, 0x400000, V2ROM);
			fclose(V2ROM);
		}

		sprintf (name, "%X%2.2X-v3d.bin", Neo68KROMActive[0x109], Neo68KROMActive[0x108]);
		FILE* V3ROM = fopen(name, "wb");

		if (V3ROM) {
			fwrite(YM2610ADPCMAROM + 0x800000, 1, 0x200000, V3ROM);
			fclose(V3ROM);
		}
	} else {
		BurnUpdateProgress(0.0, _T("Saving Decrypted V1 and V2 ROMs..."), 0);

		sprintf (name, "%X%2.2X-v1d.bin", Neo68KROMActive[0x109], Neo68KROMActive[0x108]);
		FILE* V1ROM = fopen(name, "wb");

		if (V1ROM) {
			fwrite(YM2610ADPCMAROM, 1, 0x800000, V1ROM);
			fclose(V1ROM);
		}

		sprintf (name, "%X%2.2X-v2d.bin", Neo68KROMActive[0x109], Neo68KROMActive[0x108]);
		FILE* V2ROM = fopen(name, "wb");

		if (V2ROM) {
			fwrite(YM2610ADPCMAROM + 0x800000, 1, 0x800000, V2ROM);
			fclose(V2ROM);
		}
	}
}

void SaveDecM1ROM()
{
	char name[64] = "";

	BurnUpdateProgress(0.0, _T("Saving Decrypted M1-ROM..."), 0);

	sprintf (name, "%X%2.2X-m1d.bin", Neo68KROMActive[0x109], Neo68KROMActive[0x108]);
	FILE* file = fopen(name, "wb");

	if (file) {
		fwrite(NeoZ80ROM, 1, nZ80ROMSize, file);
		fclose(file);
	}
}
